/*
 * Copyright (C) Pietro Pilolli 2010 <pilolli.pietro@gmail.com>
 * 
 * Curtain is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Curtain is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "curtain.h"
#include "callbacks.h"
#include "utils.h"


#ifndef _WIN32
/* Call the dialog that inform the user to enable a composite manager. */
static void
run_missing_composite_manager_dialog ()
{
  GtkWidget *msg_dialog;
  msg_dialog = gtk_message_dialog_new (NULL,
				       GTK_DIALOG_MODAL,
				       GTK_MESSAGE_ERROR,
				       GTK_BUTTONS_OK,
				       gettext ("In order to run Curtain you need to enable a composite manager"));

  gtk_dialog_run (GTK_DIALOG (msg_dialog));

  if (msg_dialog != NULL)
    {
      gtk_widget_destroy (msg_dialog);
      msg_dialog = NULL;
    }

  exit (EXIT_FAILURE);
}


/* Check if a composite manager is active. */
static void
check_composite_manager (GdkScreen *screen)
{
  gboolean composite = gdk_screen_is_composited (screen);

  if (!composite)
    {
      /* start the enable composite manager dialog. */
      run_missing_composite_manager_dialog ();
    }

}
#endif


/* Set the best colormap available on the system. */
static void
set_the_best_colormap ()
{
  GdkDisplay *display = gdk_display_get_default ();
  GdkScreen  *screen  = gdk_display_get_default_screen (display);
  GdkVisual  *visual  = (GdkVisual *) NULL;
  GdkColormap *colormap = gdk_screen_get_rgba_colormap (screen);

  /* In FreeBSD operating system you might have a composite manager. */
#if ( defined (__freebsd__) || defined (__freebsd) || defined (_freebsd) || defined (freebsd) )
  check_composite_manager (screen);
#endif

  /* In linux operating system you must have a composite manager. */
#ifdef linux
  check_composite_manager (screen);
#endif

  if (colormap)
    {
      gtk_widget_set_default_colormap (colormap);
    }
  else
    {
      g_warning ("The screen does not support the alpha channel\n");
      visual = gdk_screen_get_rgba_visual (screen);

      if (visual == NULL)
	{
	  g_warning ("The screen does not support the rgba visual!\n");
#ifndef _WIN32
	  exit (EXIT_FAILURE);
#endif
	}
    }

}


/* Create the window. */
GtkWidget*
create_window (void)
{
  GtkWidget *window = (GtkWidget *) NULL;
  GtkBuilder *builder = (GtkBuilder *) NULL;
  GError* error = (GError *) NULL;

  builder = gtk_builder_new ();
  if (!gtk_builder_add_from_file (builder, UI_FILE, &error))
    {
      g_warning ("Couldn't load builder file: %s", error->message);
      g_error_free (error);
    }

  window = GTK_WIDGET (gtk_builder_get_object (builder, "window"));

#ifdef _WIN32
  gtk_widget_set_double_buffered (window, FALSE);
#endif
  
  gint width = gdk_screen_width ();
  gint height = gdk_screen_height ();
   
  gtk_widget_set_usize (window, width, height);
  
  gtk_window_fullscreen (GTK_WINDOW(window)); 
    
  /* This is important; connect the gtk signals for callbacks. */
  gtk_builder_connect_signals (builder, NULL);
  g_object_unref (builder);

  return window;
}


/* Main entry of the program. */
int
main (int argc, char *argv[])
{
  GtkWidget *window;

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (GETTEXT_PACKAGE);
#endif

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  set_the_best_colormap ();

  window = create_window ();
  gtk_window_set_keep_above(GTK_WINDOW(window), TRUE);
  gtk_widget_show_all (window);

  gtk_main ();
  return 0;
}


